import { world, system } from "@minecraft/server";
import { util } from "./util.js";
import { config } from "./config.js";

world.afterEvents.worldInitialize.subscribe(ev => {
    config.load = true;
});

system.runInterval(() => {
    try{
        config.setting.nameTagColor.all = world.getDynamicProperty("namecolor_all");
        config.setting.nameTagColor.prefix = world.getDynamicProperty("namecolor_prefix");
        config.setting.nameTagColor.none = world.getDynamicProperty("namecolor_none");
    }catch{};
    for (const player of world.getAllPlayers()) util.setName(player);
});

world.beforeEvents.chatSend.subscribe(ev => {
    const { sender, message } = ev;

    if (message.startsWith("tc ")) {
        ev.cancel = true;
        system.run(() => {
            const msg = message.slice(3); // Drop "tc "
            if (msg == "on") {
                if (sender.getDynamicProperty("teamchat")) return sender.sendMessage("§a[TC] §cエラー: すでにオンです");
                sender.setDynamicProperty("teamchat",true);
                sender.sendMessage("§a[TC] チームチャットをオンにしました");
            } else if (msg == "off") {
                if (!sender.getDynamicProperty("teamchat")) return sender.sendMessage("§a[TC] §cエラー: すでにオフです");
                sender.setDynamicProperty("teamchat",false);
                sender.sendMessage("§a[TC] チームチャットをオフにしました");
            } else if (msg == "all" && sender.hasTag("op")) {
                if (world.getDynamicProperty("namecolor_all")) return sender.sendMessage("§a[TC] §cエラー: すでにオンです");
                world.setDynamicProperty("namecolor_all",true);
                world.setDynamicProperty("namecolor_prefix",false);
                world.setDynamicProperty("namecolor_none",false);
                sender.sendMessage("§a[TC] ネームカラーの設定 all に変更しました");
            } else if (msg == "prefix" && sender.hasTag("op")) {
                if (world.getDynamicProperty("namecolor_prefix")) return sender.sendMessage("§a[TC] §cエラー: すでにオンです");
                world.setDynamicProperty("namecolor_all",false);
                world.setDynamicProperty("namecolor_prefix",true);
                world.setDynamicProperty("namecolor_none",false);
                sender.sendMessage("§a[TC] ネームカラーの設定 prefix に変更しました");
            } else if (msg == "none" && sender.hasTag("op")) {
                if (world.getDynamicProperty("namecolor_none")) return sender.sendMessage("§a[TC] §cエラー: すでにオンです");
                world.setDynamicProperty("namecolor_all",false);
                world.setDynamicProperty("namecolor_prefix",false);
                world.setDynamicProperty("namecolor_none",true);
                sender.sendMessage("§a[TC] ネームカラーの設定 none に変更しました");
            };
        });
    }
    else if (util.getcolor(sender)) {
        if (message.startsWith("+")) {
            const msg =
                  sender.getDynamicProperty("teamchat")
                  ? message
                  : message.substr(1); // Drop "+"

            ev.cancel = true;
            multicastToTeam(sender, msg);
        }
        else if (message.startsWith("-")) {
            const msg =
                  sender.getDynamicProperty("teamchat")
                  ? message.substr(1) // Drop "-"
                  : message;

            ev.cancel = true;
            broadcast(sender, msg);
        }
        else if (sender.getDynamicProperty("teamchat")) {
            ev.cancel = true;
            multicastToTeam(sender, message);
        }
    }
    else {
        // Let the game process the event normally by doing nothing here.
    }
});

function broadcast(sender, message) {
    for (const player of world.getAllPlayers()) {
        player.sendMessage(`<${sender.name}> ${message}`);
    }
}

function multicastToTeam(sender, message) {
    const colors = util.getcolor(sender);
    let players = [];
    for (const player of world.getAllPlayers()) {
        for (const color of colors) {
            if (player.hasTag(color)) {
                if (players.includes(player.id)) continue;
                players.push(player.id);
                player.sendMessage(`§f<§f${`§${config.colors[color]}` + sender.name}§f> ${message}`);
            }
        }
    }
}
